@@ -36,8 +36,6 @@ uint32_t nftnl_expr_get_u32(const struct nftnl_expr *expr, uint16_t type);
 uint64_t nftnl_expr_get_u64(const struct nftnl_expr *expr, uint16_t type);
 const char *nftnl_expr_get_str(const struct nftnl_expr *expr, uint16_t type);
 
-bool nftnl_expr_cmp(const struct nftnl_expr *e1, const struct nftnl_expr *e2);
-
 int nftnl_expr_snprintf(char *buf, size_t buflen, const struct nftnl_expr *expr, uint32_t type, uint32_t flags);
 int nftnl_expr_fprintf(FILE *fp, const struct nftnl_expr *expr, uint32_t type, uint32_t flags);
 
@@ -57,6 +55,8 @@ enum {
 	NFTNL_EXPR_NG_MODULUS,
 	NFTNL_EXPR_NG_TYPE,
 	NFTNL_EXPR_NG_OFFSET,
+	NFTNL_EXPR_NG_SET_NAME,
+	NFTNL_EXPR_NG_SET_ID,
 };
 
 enum {
@@ -70,6 +70,16 @@ enum {
 	NFTNL_EXPR_RT_DREG,
 };
 
+enum {
+	NFTNL_EXPR_SOCKET_KEY	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_SOCKET_DREG,
+};
+
+enum {
+	NFTNL_EXPR_TUNNEL_KEY	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_TUNNEL_DREG,
+};
+
 enum {
 	NFTNL_EXPR_CMP_SREG	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_CMP_OP,
@@ -95,6 +105,11 @@ enum {
 	NFTNL_EXPR_CTR_BYTES,
 };
 
+enum {
+	NFTNL_EXPR_CONNLIMIT_COUNT = NFTNL_EXPR_BASE,
+	NFTNL_EXPR_CONNLIMIT_FLAGS,
+};
+
 enum {
 	NFTNL_EXPR_BITWISE_SREG	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_BITWISE_DREG,
@@ -125,6 +140,12 @@ enum {
 	NFTNL_EXPR_NAT_FLAGS,
 };
 
+enum {
+	NFTNL_EXPR_TPROXY_FAMILY	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_TPROXY_REG_ADDR,
+	NFTNL_EXPR_TPROXY_REG_PORT,
+};
+
 enum {
 	NFTNL_EXPR_LOOKUP_SREG	= NFTNL_EXPR_BASE,
 	NFTNL_EXPR_LOOKUP_DREG,
@@ -220,8 +241,14 @@ enum {
 	NFTNL_EXPR_DUP_SREG_DEV,
 };
 
+enum {
+	NFTNL_EXPR_FLOW_TABLE_NAME = NFTNL_EXPR_BASE,
+};
+
 enum {
 	NFTNL_EXPR_FWD_SREG_DEV = NFTNL_EXPR_BASE,
+	NFTNL_EXPR_FWD_SREG_ADDR,
+	NFTNL_EXPR_FWD_NFPROTO,
 };
 
 enum {
@@ -232,6 +259,8 @@ enum {
 	NFTNL_EXPR_HASH_SEED,
 	NFTNL_EXPR_HASH_OFFSET,
 	NFTNL_EXPR_HASH_TYPE,
+	NFTNL_EXPR_HASH_SET_NAME,
+	NFTNL_EXPR_HASH_SET_ID,
 };
 
 enum {
@@ -248,6 +277,19 @@ enum {
 	NFTNL_EXPR_OBJREF_SET_ID,
 };
 
+enum {
+	NFTNL_EXPR_OSF_DREG	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_OSF_TTL,
+};
+
+enum {
+	NFTNL_EXPR_XFRM_DREG	= NFTNL_EXPR_BASE,
+	NFTNL_EXPR_XFRM_SREG,
+	NFTNL_EXPR_XFRM_KEY,
+	NFTNL_EXPR_XFRM_DIR,
+	NFTNL_EXPR_XFRM_SPNUM,
+};
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
