@@ -8,6 +8,10 @@
 
 #include <libnftnl/common.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 enum nftnl_set_attr {
 	NFTNL_SET_TABLE,
 	NFTNL_SET_NAME,
@@ -24,6 +28,7 @@ enum nftnl_set_attr {
 	NFTNL_SET_GC_INTERVAL,
 	NFTNL_SET_USERDATA,
 	NFTNL_SET_OBJ_TYPE,
+	NFTNL_SET_HANDLE,
 	__NFTNL_SET_MAX
 };
 #define NFTNL_SET_MAX (__NFTNL_SET_MAX - 1)
@@ -143,4 +148,8 @@ void nftnl_set_elems_iter_destroy(struct nftnl_set_elems_iter *iter);
 int nftnl_set_elems_nlmsg_build_payload_iter(struct nlmsghdr *nlh,
 					   struct nftnl_set_elems_iter *iter);
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* _LIBNFTNL_SET_H_ */
