@@ -2,6 +2,7 @@
 #define _OBJ_OPS_H_
 
 #include <stdint.h>
+#include <libnftnl/object.h>	/* For NFTNL_CTTIMEOUT_ARRAY_MAX. */
 #include "internal.h"
 
 struct nlattr;
@@ -19,6 +20,7 @@ struct nftnl_obj {
 	uint32_t		use;
 
 	uint32_t		flags;
+	uint64_t		handle;
 
 	union {
 		struct nftnl_obj_counter {
@@ -35,6 +37,11 @@ struct nftnl_obj {
 			uint8_t		l4proto;
 			char		name[16];
 		} ct_helper;
+		struct nftnl_obj_ct_timeout {
+			uint16_t	l3proto;
+			uint8_t 	l4proto;
+			uint32_t	timeout[NFTNL_CTTIMEOUT_ARRAY_MAX];
+		} ct_timeout;
 		struct nftnl_obj_limit {
 			uint64_t	rate;
 			uint64_t	unit;
@@ -42,6 +49,37 @@ struct nftnl_obj {
 			uint32_t	type;
 			uint32_t	flags;
 		} limit;
+		struct nftnl_obj_tunnel {
+			uint32_t	id;
+			uint32_t	src_v4;
+			uint32_t	dst_v4;
+			struct in6_addr src_v6;
+			struct in6_addr dst_v6;
+			uint16_t	sport;
+			uint16_t	dport;
+			uint32_t	flowlabel;
+			uint32_t	tun_flags;
+			uint8_t		tun_tos;
+			uint8_t		tun_ttl;
+			union {
+				struct {
+					uint32_t	gbp;
+				} tun_vxlan;
+				struct {
+					uint32_t	version;
+					union {
+						uint32_t	v1_index;
+						struct {
+							uint8_t	hwid;
+							uint8_t	dir;
+						} v2;
+					} u;
+				} tun_erspan;
+			} u;
+		} tunnel;
+		struct nftnl_obj_secmark {
+			char		ctx[NFT_SECMARK_CTX_MAXLEN];
+		} secmark;
 	} data;
 };
 
@@ -55,14 +93,15 @@ struct obj_ops {
 	int	(*parse)(struct nftnl_obj *e, struct nlattr *attr);
 	void	(*build)(struct nlmsghdr *nlh, const struct nftnl_obj *e);
 	int	(*snprintf)(char *buf, size_t len, uint32_t type, uint32_t flags, const struct nftnl_obj *e);
-	int	(*json_parse)(struct nftnl_obj *e, json_t *data,
-			      struct nftnl_parse_err *err);
 };
 
 extern struct obj_ops obj_ops_counter;
 extern struct obj_ops obj_ops_quota;
 extern struct obj_ops obj_ops_ct_helper;
+extern struct obj_ops obj_ops_ct_timeout;
 extern struct obj_ops obj_ops_limit;
+extern struct obj_ops obj_ops_tunnel;
+extern struct obj_ops obj_ops_secmark;
 
 #define nftnl_obj_data(obj) (void *)&obj->data
 
