@@ -25,6 +25,10 @@ struct nftnl_expr_ng {
 	unsigned int		modulus;
 	enum nft_ng_types	type;
 	unsigned int		offset;
+	struct {
+		const char	*name;
+		uint32_t	id;
+	} map;
 };
 
 static int
@@ -35,16 +39,24 @@ nftnl_expr_ng_set(struct nftnl_expr *e, uint16_t type,
 
 	switch (type) {
 	case NFTNL_EXPR_NG_DREG:
-		ng->dreg = *((uint32_t *)data);
+		memcpy(&ng->dreg, data, sizeof(ng->dreg));
 		break;
 	case NFTNL_EXPR_NG_MODULUS:
-		ng->modulus = *((uint32_t *)data);
+		memcpy(&ng->modulus, data, sizeof(ng->modulus));
 		break;
 	case NFTNL_EXPR_NG_TYPE:
-		ng->type = *((uint32_t *)data);
+		memcpy(&ng->type, data, sizeof(ng->type));
 		break;
 	case NFTNL_EXPR_NG_OFFSET:
-		ng->offset = *((uint32_t *)data);
+		memcpy(&ng->offset, data, sizeof(ng->offset));
+		break;
+	case NFTNL_EXPR_NG_SET_NAME:
+		ng->map.name = strdup(data);
+		if (!ng->map.name)
+			return -1;
+		break;
+	case NFTNL_EXPR_NG_SET_ID:
+		memcpy(&ng->map.id, data, sizeof(ng->map.id));
 		break;
 	default:
 		return -1;
@@ -71,6 +83,12 @@ nftnl_expr_ng_get(const struct nftnl_expr *e, uint16_t type,
 	case NFTNL_EXPR_NG_OFFSET:
 		*data_len = sizeof(ng->offset);
 		return &ng->offset;
+	case NFTNL_EXPR_NG_SET_NAME:
+		*data_len = strlen(ng->map.name) + 1;
+		return ng->map.name;
+	case NFTNL_EXPR_NG_SET_ID:
+		*data_len = sizeof(ng->map.id);
+		return &ng->map.id;
 	}
 	return NULL;
 }
@@ -88,9 +106,14 @@ static int nftnl_expr_ng_cb(const struct nlattr *attr, void *data)
 	case NFTA_NG_MODULUS:
 	case NFTA_NG_TYPE:
 	case NFTA_NG_OFFSET:
+	case NFTA_NG_SET_ID:
 		if (mnl_attr_validate(attr, MNL_TYPE_U32) < 0)
 			abi_breakage();
 		break;
+	case NFTA_NG_SET_NAME:
+		if (mnl_attr_validate(attr, MNL_TYPE_STRING) < 0)
+			abi_breakage();
+		break;
 	}
 
 	tb[type] = attr;
@@ -110,6 +133,10 @@ nftnl_expr_ng_build(struct nlmsghdr *nlh, const struct nftnl_expr *e)
 		mnl_attr_put_u32(nlh, NFTA_NG_TYPE, htonl(ng->type));
 	if (e->flags & (1 << NFTNL_EXPR_NG_OFFSET))
 		mnl_attr_put_u32(nlh, NFTA_NG_OFFSET, htonl(ng->offset));
+	if (e->flags & (1 << NFTNL_EXPR_NG_SET_NAME))
+		mnl_attr_put_str(nlh, NFTA_NG_SET_NAME, ng->map.name);
+	if (e->flags & (1 << NFTNL_EXPR_NG_SET_ID))
+		mnl_attr_put_u32(nlh, NFTA_NG_SET_ID, htonl(ng->map.id));
 }
 
 static int
@@ -138,39 +165,20 @@ nftnl_expr_ng_parse(struct nftnl_expr *e, struct nlattr *attr)
 		ng->offset = ntohl(mnl_attr_get_u32(tb[NFTA_NG_OFFSET]));
 		e->flags |= (1 << NFTNL_EXPR_NG_OFFSET);
 	}
+	if (tb[NFTA_NG_SET_NAME]) {
+		ng->map.name =
+			strdup(mnl_attr_get_str(tb[NFTA_NG_SET_NAME]));
+		e->flags |= (1 << NFTNL_EXPR_NG_SET_NAME);
+	}
+	if (tb[NFTA_NG_SET_ID]) {
+		ng->map.id =
+			ntohl(mnl_attr_get_u32(tb[NFTA_NG_SET_ID]));
+		e->flags |= (1 << NFTNL_EXPR_NG_SET_ID);
+	}
 
 	return ret;
 }
 
-static int nftnl_expr_ng_json_parse(struct nftnl_expr *e, json_t *root,
-				    struct nftnl_parse_err *err)
-{
-#ifdef JSON_PARSING
-	uint32_t dreg, modulus, type, offset;
-
-	if (nftnl_jansson_parse_reg(root, "dreg", NFTNL_TYPE_U32,
-				    &dreg, err) == 0)
-		nftnl_expr_set_u32(e, NFTNL_EXPR_NG_DREG, dreg);
-
-	if (nftnl_jansson_parse_val(root, "modulus", NFTNL_TYPE_U32,
-				    &modulus, err) == 0)
-		nftnl_expr_set_u32(e, NFTNL_EXPR_NG_MODULUS, modulus);
-
-	if (nftnl_jansson_parse_val(root, "type", NFTNL_TYPE_U32,
-				    &type, err) == 0)
-		nftnl_expr_set_u32(e, NFTNL_EXPR_NG_TYPE, type);
-
-	if (nftnl_jansson_parse_val(root, "offset", NFTNL_TYPE_U32,
-				    &offset, err) == 0)
-		nftnl_expr_set_u32(e, NFTNL_EXPR_NG_OFFSET, offset);
-
-	return 0;
-#else
-	errno = EOPNOTSUPP;
-	return -1;
-#endif
-}
-
 static int
 nftnl_expr_ng_snprintf_default(char *buf, size_t size,
 			       const struct nftnl_expr *e)
@@ -198,28 +206,15 @@ nftnl_expr_ng_snprintf_default(char *buf, size_t size,
 		SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 	}
 
+	if (ng->map.id) {
+		ret = snprintf(buf + offset, remain, "set %s id %u ",
+			       ng->map.name, ng->map.id);
+		SNPRINTF_BUFFER_SIZE(ret, remain, offset);
+	}
+
 	return offset;
 }
 
-static int nftnl_expr_ng_export(char *buf, size_t size,
-				const struct nftnl_expr *e, int type)
-{
-	struct nftnl_expr_ng *ng = nftnl_expr_data(e);
-
-	NFTNL_BUF_INIT(b, buf, size);
-
-	if (e->flags & (1 << NFTNL_EXPR_NG_DREG))
-		nftnl_buf_u32(&b, type, ng->dreg, DREG);
-	if (e->flags & (1 << NFTNL_EXPR_NG_MODULUS))
-		nftnl_buf_u32(&b, type, ng->modulus, MODULUS);
-	if (e->flags & (1 << NFTNL_EXPR_NG_TYPE))
-		nftnl_buf_u32(&b, type, ng->type, TYPE);
-	if (e->flags & (1 << NFTNL_EXPR_NG_OFFSET))
-		nftnl_buf_u32(&b, type, ng->type, OFFSET);
-
-	return nftnl_buf_done(&b);
-}
-
 static int
 nftnl_expr_ng_snprintf(char *buf, size_t len, uint32_t type,
 		       uint32_t flags, const struct nftnl_expr *e)
@@ -229,41 +224,19 @@ nftnl_expr_ng_snprintf(char *buf, size_t len, uint32_t type,
 		return nftnl_expr_ng_snprintf_default(buf, len, e);
 	case NFTNL_OUTPUT_XML:
 	case NFTNL_OUTPUT_JSON:
-		return nftnl_expr_ng_export(buf, len, e, type);
 	default:
 		break;
 	}
 	return -1;
 }
 
-static bool nftnl_expr_ng_cmp(const struct nftnl_expr *e1,
-			      const struct nftnl_expr *e2)
-{
-	struct nftnl_expr_ng *n1 = nftnl_expr_data(e1);
-	struct nftnl_expr_ng *n2 = nftnl_expr_data(e2);
-	bool eq = true;
-
-	if (e1->flags & (1 << NFTNL_EXPR_NG_DREG))
-		eq &= (n1->dreg == n2->dreg);
-	if (e1->flags & (1 << NFTNL_EXPR_NG_MODULUS))
-		eq &= (n1->modulus == n2->modulus);
-	if (e1->flags & (1 << NFTNL_EXPR_NG_TYPE))
-		eq &= (n1->type == n2->type);
-	if (e1->flags & (1 << NFTNL_EXPR_NG_OFFSET))
-		eq &= (n1->offset == n2->offset);
-
-	return eq;
-}
-
 struct expr_ops expr_ops_ng = {
 	.name		= "numgen",
 	.alloc_len	= sizeof(struct nftnl_expr_ng),
 	.max_attr	= NFTA_NG_MAX,
-	.cmp		= nftnl_expr_ng_cmp,
 	.set		= nftnl_expr_ng_set,
 	.get		= nftnl_expr_ng_get,
 	.parse		= nftnl_expr_ng_parse,
 	.build		= nftnl_expr_ng_build,
 	.snprintf	= nftnl_expr_ng_snprintf,
-	.json_parse	= nftnl_expr_ng_json_parse,
 };
