@@ -32,6 +32,7 @@ struct nftnl_table {
 	const char	*name;
 	uint32_t	family;
 	uint32_t	table_flags;
+	uint64_t 	handle;
 	uint32_t	use;
 	uint32_t	flags;
 };
@@ -68,6 +69,7 @@ void nftnl_table_unset(struct nftnl_table *t, uint16_t attr)
 		xfree(t->name);
 		break;
 	case NFTNL_TABLE_FLAGS:
+	case NFTNL_TABLE_HANDLE:
 	case NFTNL_TABLE_FAMILY:
 		break;
 	case NFTNL_TABLE_USE:
@@ -79,6 +81,7 @@ void nftnl_table_unset(struct nftnl_table *t, uint16_t attr)
 static uint32_t nftnl_table_validate[NFTNL_TABLE_MAX + 1] = {
 	[NFTNL_TABLE_FLAGS]	= sizeof(uint32_t),
 	[NFTNL_TABLE_FAMILY]	= sizeof(uint32_t),
+	[NFTNL_TABLE_HANDLE]	= sizeof(uint64_t),
 };
 
 EXPORT_SYMBOL(nftnl_table_set_data);
@@ -97,14 +100,17 @@ int nftnl_table_set_data(struct nftnl_table *t, uint16_t attr,
 		if (!t->name)
 			return -1;
 		break;
+	case NFTNL_TABLE_HANDLE:
+		memcpy(&t->handle, data, sizeof(t->handle));
+		break;
 	case NFTNL_TABLE_FLAGS:
-		t->table_flags = *((uint32_t *)data);
+		memcpy(&t->table_flags, data, sizeof(t->table_flags));
 		break;
 	case NFTNL_TABLE_FAMILY:
-		t->family = *((uint32_t *)data);
+		memcpy(&t->family, data, sizeof(t->family));
 		break;
 	case NFTNL_TABLE_USE:
-		t->use = *((uint32_t *)data);
+		memcpy(&t->use, data, sizeof(t->use));
 		break;
 	}
 	t->flags |= (1 << attr);
@@ -123,6 +129,12 @@ void nftnl_table_set_u32(struct nftnl_table *t, uint16_t attr, uint32_t val)
 	nftnl_table_set_data(t, attr, &val, sizeof(uint32_t));
 }
 
+EXPORT_SYMBOL(nftnl_table_set_u64);
+void nftnl_table_set_u64(struct nftnl_table *t, uint16_t attr, uint64_t val)
+{
+	nftnl_table_set_data(t, attr, &val, sizeof(uint64_t));
+}
+
 EXPORT_SYMBOL(nftnl_table_set_u8);
 void nftnl_table_set_u8(struct nftnl_table *t, uint16_t attr, uint8_t val)
 {
@@ -146,6 +158,9 @@ const void *nftnl_table_get_data(const struct nftnl_table *t, uint16_t attr,
 	case NFTNL_TABLE_NAME:
 		*data_len = strlen(t->name) + 1;
 		return t->name;
+	case NFTNL_TABLE_HANDLE:
+		*data_len = sizeof(uint64_t);
+		return &t->handle;
 	case NFTNL_TABLE_FLAGS:
 		*data_len = sizeof(uint32_t);
 		return &t->table_flags;
@@ -173,6 +188,13 @@ uint32_t nftnl_table_get_u32(const struct nftnl_table *t, uint16_t attr)
 	return ret == NULL ? 0 : *((uint32_t *)ret);
 }
 
+EXPORT_SYMBOL(nftnl_table_get_u64);
+uint64_t nftnl_table_get_u64(const struct nftnl_table *t, uint16_t attr)
+{
+	const void *ret = nftnl_table_get(t, attr);
+	return ret == NULL ? 0 : *((uint64_t *)ret);
+}
+
 EXPORT_SYMBOL(nftnl_table_get_u8);
 uint8_t nftnl_table_get_u8(const struct nftnl_table *t, uint16_t attr)
 {
@@ -191,6 +213,8 @@ void nftnl_table_nlmsg_build_payload(struct nlmsghdr *nlh, const struct nftnl_ta
 {
 	if (t->flags & (1 << NFTNL_TABLE_NAME))
 		mnl_attr_put_strz(nlh, NFTA_TABLE_NAME, t->name);
+	if (t->flags & (1 << NFTNL_TABLE_HANDLE))
+		mnl_attr_put_u64(nlh, NFTA_TABLE_HANDLE, htobe64(t->handle));
 	if (t->flags & (1 << NFTNL_TABLE_FLAGS))
 		mnl_attr_put_u32(nlh, NFTA_TABLE_FLAGS, htonl(t->table_flags));
 }
@@ -208,6 +232,10 @@ static int nftnl_table_parse_attr_cb(const struct nlattr *attr, void *data)
 		if (mnl_attr_validate(attr, MNL_TYPE_STRING) < 0)
 			abi_breakage();
 		break;
+	case NFTA_TABLE_HANDLE:
+		if (mnl_attr_validate(attr, MNL_TYPE_U64) < 0)
+			abi_breakage();
+		break;
 	case NFTA_TABLE_FLAGS:
 	case NFTA_TABLE_USE:
 		if (mnl_attr_validate(attr, MNL_TYPE_U32) < 0)
@@ -244,6 +272,10 @@ int nftnl_table_nlmsg_parse(const struct nlmsghdr *nlh, struct nftnl_table *t)
 		t->use = ntohl(mnl_attr_get_u32(tb[NFTA_TABLE_USE]));
 		t->flags |= (1 << NFTNL_TABLE_USE);
 	}
+	if (tb[NFTA_TABLE_HANDLE]) {
+		t->handle = be64toh(mnl_attr_get_u64(tb[NFTA_TABLE_HANDLE]));
+		t->flags |= (1 << NFTNL_TABLE_HANDLE);
+	}
 
 	t->family = nfg->nfgen_family;
 	t->flags |= (1 << NFTNL_TABLE_FAMILY);
@@ -251,72 +283,14 @@ int nftnl_table_nlmsg_parse(const struct nlmsghdr *nlh, struct nftnl_table *t)
 	return 0;
 }
 
-#ifdef JSON_PARSING
-int nftnl_jansson_parse_table(struct nftnl_table *t, json_t *tree,
-			    struct nftnl_parse_err *err)
-{
-	json_t *root;
-	uint32_t flags, use;
-	const char *str;
-	int family;
-
-	root = nftnl_jansson_get_node(tree, "table", err);
-	if (root == NULL)
-		return -1;
-
-	str = nftnl_jansson_parse_str(root, "name", err);
-	if (str != NULL)
-		nftnl_table_set_str(t, NFTNL_TABLE_NAME, str);
-
-	if (nftnl_jansson_parse_family(root, &family, err) == 0)
-		nftnl_table_set_u32(t, NFTNL_TABLE_FAMILY, family);
-
-	if (nftnl_jansson_parse_val(root, "flags", NFTNL_TYPE_U32, &flags,
-				  err) == 0)
-		nftnl_table_set_u32(t, NFTNL_TABLE_FLAGS, flags);
-
-	if (nftnl_jansson_parse_val(root, "use", NFTNL_TYPE_U32, &use, err) == 0)
-		nftnl_table_set_u32(t, NFTNL_TABLE_USE, use);
-
-	return 0;
-}
-#endif
-
-static int nftnl_table_json_parse(struct nftnl_table *t, const void *json,
-				struct nftnl_parse_err *err,
-				enum nftnl_parse_input input)
-{
-#ifdef JSON_PARSING
-	json_t *tree;
-	json_error_t error;
-	int ret;
-
-	tree = nftnl_jansson_create_root(json, &error, err, input);
-	if (tree == NULL)
-		return -1;
-
-	ret = nftnl_jansson_parse_table(t, tree, err);
-
-	nftnl_jansson_free_root(tree);
-
-	return ret;
-#else
-	errno = EOPNOTSUPP;
-	return -1;
-#endif
-}
-
 static int nftnl_table_do_parse(struct nftnl_table *t, enum nftnl_parse_type type,
 			      const void *data, struct nftnl_parse_err *err,
 			      enum nftnl_parse_input input)
 {
 	int ret;
-	struct nftnl_parse_err perr = {};
 
 	switch (type) {
 	case NFTNL_PARSE_JSON:
-		ret = nftnl_table_json_parse(t, data, &perr, input);
-		break;
 	case NFTNL_PARSE_XML:
 	default:
 		ret = -1;
@@ -324,9 +298,6 @@ static int nftnl_table_do_parse(struct nftnl_table *t, enum nftnl_parse_type typ
 		break;
 	}
 
-	if (err != NULL)
-		*err = perr;
-
 	return ret;
 }
 
@@ -344,32 +315,12 @@ int nftnl_table_parse_file(struct nftnl_table *t, enum nftnl_parse_type type,
 	return nftnl_table_do_parse(t, type, fp, err, NFTNL_PARSE_FILE);
 }
 
-static int nftnl_table_export(char *buf, size_t size,
-			      const struct nftnl_table *t, int type)
-{
-	NFTNL_BUF_INIT(b, buf, size);
-
-	nftnl_buf_open(&b, type, TABLE);
-	if (t->flags & (1 << NFTNL_TABLE_NAME))
-		nftnl_buf_str(&b, type, t->name, NAME);
-	if (t->flags & (1 << NFTNL_TABLE_FAMILY))
-		nftnl_buf_str(&b, type, nftnl_family2str(t->family), FAMILY);
-	if (t->flags & (1 << NFTNL_TABLE_FLAGS))
-		nftnl_buf_u32(&b, type, t->table_flags, FLAGS);
-	if (t->flags & (1 << NFTNL_TABLE_USE))
-		nftnl_buf_u32(&b, type, t->use, USE);
-
-	nftnl_buf_close(&b, type, TABLE);
-
-	return nftnl_buf_done(&b);
-}
-
 static int nftnl_table_snprintf_default(char *buf, size_t size,
 					const struct nftnl_table *t)
 {
-	return snprintf(buf, size, "table %s %s flags %x use %d",
+	return snprintf(buf, size, "table %s %s flags %x use %d handle %llu",
 			t->name, nftnl_family2str(t->family),
-			t->table_flags, t->use);
+			t->table_flags, t->use, (unsigned long long)t->handle);
 }
 
 static int nftnl_table_cmd_snprintf(char *buf, size_t size,
@@ -378,24 +329,16 @@ static int nftnl_table_cmd_snprintf(char *buf, size_t size,
 {
 	int ret, remain = size, offset = 0;
 
-	ret = nftnl_cmd_header_snprintf(buf + offset, remain, cmd, type, flags);
-	SNPRINTF_BUFFER_SIZE(ret, remain, offset);
-
 	switch (type) {
 	case NFTNL_OUTPUT_DEFAULT:
 		ret = nftnl_table_snprintf_default(buf + offset, remain, t);
+		SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 		break;
 	case NFTNL_OUTPUT_XML:
 	case NFTNL_OUTPUT_JSON:
-		ret = nftnl_table_export(buf + offset, remain, t, type);
-		break;
 	default:
 		return -1;
 	}
-	SNPRINTF_BUFFER_SIZE(ret, remain, offset);
-
-	ret = nftnl_cmd_footer_snprintf(buf + offset, remain, cmd, type, flags);
-	SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 
 	return offset;
 }
